namespace DemoDataAccessNS
{
	using System;
	using System.Data;
	using System.Data.Common;
	using System.Data.OleDb;
	using System.Data.SqlClient;

	public class DemoDataAccess
	{
		protected const string s_JetCxnString = 
			@"Provider=Microsoft.JET.OLEDB.4.0; Data Source=..\..\Northwind.mdb";
		protected const string s_SQLServerCxnString = 
			@"Data Source=.;Initial Catalog=Northwind; user id=sa; password=zippY24";

		public static int Main(string[] args)
		{
			DemoDataReader();
			DemoStoredProcedure();
			DemoDataSet();
			DemoDataSetCreation();
			DemoUpdateDataSet2();
			DemoException();
			DemoInsert();
			
			Console.ReadLine();
			return 0;
		}

		private static void BeginDemo(string s)
		{
			Console.WriteLine(s);
		}
		
		private static void EndDemo()
		{
			Console.WriteLine("");
//			Console.ReadLine();
		}
		
		static void DemoInsert()
		{
			BeginDemo("Running Insert Sample...");
			try
			{
				// Open connection to Jet (Access) database
				OleDbConnection cxn = new OleDbConnection(s_JetCxnString);
				cxn.Open();
				
				// Make query string and Create command object
				string queryString = 
					"INSERT INTO Employees(FirstName, LastName) "
					+ "VALUES ('abc', '" + DateTime.Now.ToString() + "')";
				OleDbCommand cmdDb = new OleDbCommand(queryString, cxn);
				
				// Execute the query
				int nRows = cmdDb.ExecuteNonQuery();	
				Console.WriteLine("nRows = " + nRows);

				// Close the connection
				cxn.Close();
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception, msg = " + e.Message);
			}	
			EndDemo();
		}

		static void DemoDataSet()
		{
			BeginDemo("Running DataSet Access Database Sample...");
			try
			{
				// Open connection to Jet (Access) database
				OleDbConnection cxn = new OleDbConnection(s_JetCxnString);
				cxn.Open();

				// Create command objects
				string queryString = "SELECT FirstName, LastName FROM Employees";
				OleDbDataAdapter da = new OleDbDataAdapter();
				da.SelectCommand = new OleDbCommand(queryString, cxn);
				
				// Create and populate DataSet
				DataSet ds = new DataSet();
//				ds.Tables.Add("Employees");
				da.Fill(ds, "Employees");
				cxn.Close();
				
				// Iterate over all rows in the result
				DataTable dt = ds.Tables["Employees"];
				foreach(DataRow dr in dt.Rows)
				{
					Console.WriteLine("FirstName, LastName = " + dr["FirstName"] + ", " + dr["LastName"]);
				}
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception, msg = " + e.Message);
			}	
			EndDemo();
		}

		static void DemoDataReader()
		{
			BeginDemo("Running DataReader Access Database Sample...");
			try
			{
				// Open connection to Jet (Access) database
				OleDbConnection cxn = new OleDbConnection(s_JetCxnString);
				cxn.Open();
				
				// Make query string and Create command object
				string queryString = "SELECT FirstName + ' ' + LastName FROM Employees";
				OleDbCommand cmdDb = new OleDbCommand(queryString, cxn);
				
				// Create Data Reader
				OleDbDataReader myReader;
				myReader = cmdDb.ExecuteReader(CommandBehavior.CloseConnection);	
				
				// Iterate over all rows in the result
				while (myReader.Read())
				{
					Console.WriteLine("FirstName & LastName = " + myReader.GetString(0));
				} 
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception, msg = " + e.Message);
			}	
			EndDemo();
		}

		static void DemoException()
		{
			BeginDemo("Running Exception Sample...");
			try
			{
				Console.WriteLine("In Try{}");
				
				// Create a Table
				DataTable dt = new DataTable();
				
				// Add two columns with same name - this generates an exception and immediately falls into Catch(){}
				dt.Columns.Add("myCol");
				dt.Columns.Add("myCol");
				Console.WriteLine("We never get to this line!");
			}
			catch (DataException e)
			{	
				Console.WriteLine("In Catch(){}");
				Console.WriteLine("Sample Exception,\n msg = " + e.Message + 
					"\n src = " + e.Source + 
					"\n Stack Trace =\n" + e.StackTrace);
			}
			EndDemo();
		}

		static void DemoUpdateDataSet()
		{
			BeginDemo("Running Update DataSet Sample...");
			try
			{
				// Open connection to Jet (Access) database
				OleDbConnection cxn = new OleDbConnection(s_JetCxnString);
				cxn.Open();
				
				// Create command objects
				string queryString = "SELECT EmployeeID, firstname, lastname FROM Employees";
				OleDbDataAdapter da = new OleDbDataAdapter();
				da.SelectCommand = new OleDbCommand(queryString, cxn);
				OleDbCommandBuilder cb = new OleDbCommandBuilder(da);

				// Create and populate DataSet
				DataSet ds = new DataSet();
				//				ds.Tables.Add("Employees");
				da.Fill(ds, "Employees");

				// Change the value of the first column of first row
				DataTable dt = ds.Tables["Employees"];
				DataRow dr = dt.Rows[0];
				Console.WriteLine("Initial value: {0}", dr["FirstName"]);
				dr["FirstName"] = DateTime.Now.ToString();
				int i = da.Update(ds, "Employees");
				//				drUpdateRow.AcceptChanges();
				//				ds.AcceptChanges();
				Console.WriteLine("Updated {0} rows", i);
				cxn.Close();
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception, msg = " + e.Message);
			}	
			EndDemo();
		}

		static void DemoUpdateDataSet2()
		{
			BeginDemo("Running Update DataSet2 Sample...");
			try
			{
//				string tableName = "Region";
//				string columnName = "RegionDescription";
				string tableName = "Employees";
				string keyName = "EmployeeID";
				string columnName = "LastName";
				// Open connection to SQL Server database
				SqlConnection cxn = new SqlConnection(s_SQLServerCxnString);
				cxn.Open();
				
				// Create command objects
				string queryString = "SELECT " + keyName + "," + columnName + " FROM " + tableName;
				SqlDataAdapter da = new SqlDataAdapter();
				da.SelectCommand = new SqlCommand(queryString, cxn);
				SqlCommandBuilder cb = new SqlCommandBuilder(da);

				// Create and populate DataSet
				DataSet ds = new DataSet();
				//				ds.Tables.Add(tableName);
				da.Fill(ds, tableName);

				// Change the value of the first column of first row
				DataTable dt = ds.Tables[tableName];
				DataRow dr = dt.Rows[0];
				Console.WriteLine("Initial value: {0}", dr[columnName]);
				dr[columnName] = DateTime.Now.ToString();
//				ds.AcceptChanges();
				int i = da.Update(ds, tableName);
				//				drUpdateRow.AcceptChanges();
				Console.WriteLine("Updated {0} rows", i);
				cxn.Close();
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception:{0}", e);
			}	
			EndDemo();
		}

		static void DemoStoredProcedure()
		{
			BeginDemo("Running Stored Procedure Sample...");
			try
			{
				// Open connection to Sql Server database
				SqlConnection cxn = new SqlConnection(s_SQLServerCxnString);
				cxn.Open();
				
				// Create command objects
				string queryString = "Prc_GetEmployeeById " + 1;
				SqlDataAdapter da = new SqlDataAdapter();
				da.SelectCommand = new SqlCommand(queryString, cxn);

				// Create and populate DataSet
				DataSet ds = new DataSet();
				ds.Tables.Add("Employees");
				da.Fill(ds, "Employees");

				// Pull out the Employee table and display data
				DataTable dt = ds.Tables["Employees"];
				Console.WriteLine("# rows in result: {0}", dt.Rows.Count);
				DataRow drEmployee = dt.Rows[0];
				cxn.Close();
				Console.WriteLine("FirstName, LastName: Id = " + drEmployee["FirstName"] + ", " + drEmployee["LastName"] + ": " + drEmployee["EmployeeId"]);
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception, msg = " + e.Message);
			}	
			EndDemo();
		}

		static void DemoDataSetCreation()
		{
			BeginDemo("Running DataSet Creation Sample...");
			try
			{
				//Create and name DataSet
				DataSet dataset = new DataSet();
				dataset.DataSetName = "BookAuthors";
				
				//Create tables
				DataTable authors = new DataTable("Author");
				DataTable books = new DataTable("Book");
				
				//Add columns to tables and modify the table schema
				DataColumn id = authors.Columns.Add("ID", typeof(Int32));
				id.AutoIncrement = true;
				authors.PrimaryKey = new DataColumn[] {id};
				DataColumn name = authors.Columns.Add("Name", typeof(String));
				DataColumn isbn = books.Columns.Add("ISBN", typeof(String));
				books.PrimaryKey = new DataColumn[] {isbn};
				DataColumn title = books.Columns.Add("Title", typeof(String));
				DataColumn authid = books.Columns.Add("AuthID",typeof(Int32));
				DataColumn[] foreignkey = new DataColumn[] {authid};
				
				//Add tables to the new dataset	
				dataset.Tables.Add(authors);
				dataset.Tables.Add(books);
				
				//create and populate a row
				DataRow shkspr = authors.NewRow();
				shkspr["Name"] = "William Shakespeare";
				
				//Add the row to the authors table
				authors.Rows.Add(shkspr);
				
				//Define relationship schema
				DataRelation bookauth = new DataRelation("BookAuthors", authors.PrimaryKey, foreignkey);
				dataset.Relations.Add (bookauth);
				DataRow row = books.NewRow();
				row["AuthID"] = shkspr["ID"];
				row["ISBN"] = "1000-XYZ";
				row["Title"] = "MacBeth";
				books.Rows.Add(row);
				
				//commit changes in the dataset
				dataset.AcceptChanges();
				Console.WriteLine("Tables created in dataset: " + dataset.Tables.Count);      
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception, msg = " + e.Message);
			}	
			EndDemo();
		}
	}
}
